@@ -1,21 +1,24 @@
 use strict;
 use warnings;
-package Dist::Zilla::Plugin::DynamicPrereqs; # git description: v0.020-5-g167d1d4
+package Dist::Zilla::Plugin::DynamicPrereqs; # git description: v0.023-3-g59b8951
 # vim: set ts=8 sts=4 sw=4 tw=115 et :
 # ABSTRACT: Specify dynamic (user-side) prerequisites for your distribution
 # KEYWORDS: plugin distribution metadata MYMETA prerequisites Makefile.PL dynamic
 
-our $VERSION = '0.021'; # TRIAL
+our $VERSION = '0.024'; # TRIAL
 
 use Moose;
 with
+    'Dist::Zilla::Role::FileGatherer',
+    'Dist::Zilla::Role::ModuleIncluder',
+    'Dist::Zilla::Role::ModuleMetadata',
     'Dist::Zilla::Role::FileMunger',
     'Dist::Zilla::Role::MetaProvider',
     'Dist::Zilla::Role::PrereqSource',
     'Dist::Zilla::Role::AfterBuild',
     'Dist::Zilla::Role::TextTemplate',
 ;
-use List::Util 1.33 qw(first notall any);
+use List::Util 1.45 qw(first notall any uniq);
 use Module::Runtime 'module_notional_filename';
 use Try::Tiny;
 use Path::Tiny;
@@ -228,11 +231,8 @@ sub _build__sub_definitions
     return $content;
 }
 
+
 my %sub_prereqs = (
-    can_xs => {
-        'ExtUtils::CBuilder' => '0.27',
-        'File::Temp' => '0',
-    },
     can_cc => {
         'Config' => '0',
     },
@@ -249,19 +249,70 @@ my %sub_prereqs = (
     },
 );
 
+# instead of including these dependencies in configure-requires, we inline
+# them right into the distribution in inc/.
+my %sub_inc_dependencies = (
+    can_xs => {
+        'ExtUtils::HasCompiler' => '0.013',
+    },
+);
+
+sub gather_files {
+    my $self = shift;
+
+    foreach my $required_sub ($self->_all_required_subs)
+    {
+        # FIXME: update %included_subs earlier to account for other coexisting
+        # plugins, or running two instances of the plugin will try to do this twice.
+        my $include_modules = $sub_inc_dependencies{$required_sub} || {};
+        foreach my $module (keys %$include_modules)
+        {
+            (my $path = $module) =~ s{::}{/}g;
+            $path = path('inc', $path . '.pm');
+
+            my $file = (first { $_->name eq $path } @{ $self->zilla->files })
+                # TODO this requires a new release of ModuleIncluder
+                # || ($self->include_modules([ $module ], '5.006'))[0];
+                || do {
+                    $self->log([ 'inlining %s into inc/', $module ]);
+                    $self->include_modules([ $module ], version->new('5.006'));
+                    first { $_->name eq $path } @{ $self->zilla->files };
+                };
+
+            if (defined $include_modules->{$module} and $include_modules->{$module} > 0)
+            {
+                my $mmd = $self->module_metadata_for_file($file);
+                $self->log_fatal([ '%s version %s required--only found version %s',
+                        $module, $include_modules->{$module},
+                        (defined $mmd->version ? $mmd->version->stringify : 'undef') ])
+                    if ($mmd->version || 0) < $include_modules->{$module};
+            }
+        }
+    }
+}
+
 sub register_prereqs
 {
     my $self = shift;
     foreach my $required_sub ($self->_all_required_subs)
     {
-        my $prereqs = $sub_prereqs{$required_sub};
+        my $configure_prereqs = $sub_prereqs{$required_sub} || {};
         $self->zilla->register_prereqs(
             {
                 phase => 'configure',
                 type  => 'requires',
             },
-            %$prereqs,
-        ) if $prereqs and %$prereqs;
+            %$configure_prereqs,
+        ) if %$configure_prereqs;
+
+        my $develop_prereqs = $sub_inc_dependencies{$required_sub} || {};
+        $self->zilla->register_prereqs(
+            {
+                phase => 'develop',
+                type  => 'requires',
+            },
+            %$develop_prereqs,
+        ) if %$develop_prereqs;
     }
 }
 
@@ -276,13 +327,13 @@ has _include_sub_root => (
 
 # indicates subs that require other subs to be included
 my %sub_dependencies = (
-    can_xs => [ qw(can_cc) ],
     can_cc => [ qw(can_run) ],
     can_run => [ qw(maybe_command) ],
     requires => [ qw(runtime_requires) ],
     runtime_requires => [ qw(_add_prereq) ],
     build_requires => [ qw(_add_prereq) ],
     test_requires => [ qw(_add_prereq) ],
+    want_pp => [ qw(parse_args) ],
 );
 
 has _all_required_subs => (
@@ -299,7 +350,7 @@ has _all_required_subs => (
                 any { /\b$sub_name\b\(/ } @code
             } map { $_->basename } path($self->_include_sub_root)->children;
 
-        [ sort($self->_all_required_subs_for(_uniq(
+        [ sort($self->_all_required_subs_for(uniq(
             $self->include_subs, @subs_in_code,
         ))) ];
     },
@@ -336,8 +387,6 @@ sub _warn_include_subs
         foreach grep { exists $warn_include_sub{$_} } @include_subs;
 }
 
-sub _uniq { keys %{ +{ map { $_ => undef } @_ } } }
-
 __PACKAGE__->meta->make_immutable;
 
 __END__
@@ -352,7 +401,7 @@ Dist::Zilla::Plugin::DynamicPrereqs - Specify dynamic (user-side) prerequisites
 
 =head1 VERSION
 
-version 0.021
+version 0.024
 
 =head1 SYNOPSIS
 
@@ -360,7 +409,7 @@ In your F<dist.ini>:
 
     [DynamicPrereqs]
     -condition = has_module('Role::Tiny')
-    -condition = !parse_args()->{PUREPERL_ONLY}
+    -condition = !want_pp()
     -condition = can_xs()
     -body = requires('Role::Tiny', '1.003000')
 
@@ -430,7 +479,7 @@ C<[Prereqs / ConfigureRequires]> in your F<dist.ini>).
 
 C<-body> first became available in version 0.018.
 
-=for Pod::Coverage mvp_multivalue_args mvp_aliases BUILD metadata after_build munge_files register_prereqs
+=for Pod::Coverage mvp_multivalue_args mvp_aliases BUILD metadata after_build munge_files register_prereqs gather_files
 
 =head2 C<-delimiter>
 
@@ -513,7 +562,7 @@ C<parse_args()> - returns the hashref of options that were passed as arguments t
 
 =item *
 
-C<can_xs()> - Secondary compile testing via L<ExtUtils::CBuilder>
+C<can_xs()> - XS capability testing via L<ExtUtils::HasCompiler> (don't forget to also check C<want_pp>!)
 
 =item *
 
@@ -571,6 +620,10 @@ C<build_requires($module [, $version ])> - adds the module to runtime prereqs (a
 
 C<test_requires($module [, $version ])> - adds the module to runtime prereqs (as a shorthand for editing the hashes in F<Makefile.PL> directly). Added in 0.016.
 
+=item *
+
+C<want_pp> - true if the user or CPAN client explicitly specified PUREPERL_ONLY (indicating that no XS-requiring modules or code should be installed)
+
 =back
 
 =head1 WARNING: INCOMPLETE SUBROUTINE IMPLEMENTATIONS!
